@@ -56,6 +56,7 @@ from aiohomekit.uuid import normalize_uuid
 from ..abstract import AbstractPairing, AbstractPairingData
 from .bleak import BLEAK_EXCEPTIONS, AIOHomeKitBleakClient
 from .client import (
+    PDUStatusError,
     ble_request,
     drive_pairing_state_machine,
     raise_for_pdu_status,
@@ -90,6 +91,14 @@ SKIP_SYNC_SERVICES = {
     ServicesTypes.PAIRING,
     ServicesTypes.TRANSFER_TRANSPORT_MANAGEMENT,
 }
+# These characteristics are not readable unless there has been a write
+WRITE_FIRST_REQUIRED_CHARACTERISTICS = {
+    "00000131-0000-1000-8000-0026BB765291",  # Setup Data Stream Transport
+    "00000117-0000-1000-8000-0026BB765291",  # Selected RTP Stream Configuration
+    "00000118-0000-1000-8000-0026BB765291",  # Setup Endpoints
+    "00000138-0000-1000-8000-0026BB765291",  # Unknown write first characteristic
+    "246912DC-8FA3-82ED-DEA4-9EB91D8FC2EE",  # Unknown Vendor char seen on Belkin Wemo Switch
+}
 BLE_AID = 1  # The aid for BLE devices is always 1
 
 WrapFuncType = TypeVar("WrapFuncType", bound=Callable[..., Any])
@@ -121,18 +130,17 @@ class BlePairing(AbstractPairing):
         client: AIOHomeKitBleakClient | None = None,
         description: HomeKitAdvertisement | None = None,
     ) -> None:
-        super().__init__(controller)
-
-        self.id = pairing_data["AccessoryPairingID"]
         self.device = device
         self.client = client
+        self.description = description
+        self.pairing_data = pairing_data
+
+        super().__init__(controller, pairing_data)
+
         self._cached_services: BleakGATTServiceCollection | None = (
             client.services if client else None
         )
 
-        self.pairing_data = pairing_data
-        self.description = description
-        self.controller = controller
         self._last_seen = time.monotonic() if description else NEVER_TIME
 
         # Encryption
@@ -174,10 +182,10 @@ class BlePairing(AbstractPairing):
 
     @property
     def name(self) -> str:
-        """Return the name of the pairing."""
+        """Return the name of the pairing with the address."""
         if self.description:
-            return f"{self.description.name} ({self.address})"
-        return self.address
+            return f"{self.description.name} [{self.address}] (id={self.id})"
+        return f"[{self.address}] (id={self.id})"
 
     @property
     def rssi(self) -> int | None:
@@ -211,15 +219,31 @@ class BlePairing(AbstractPairing):
 
     def _async_ble_device_update(self, device: BLEDevice) -> None:
         """Update the BLE device."""
-        if self.device and ble_device_has_changed(self.device, device):
-            self._cached_services = None
-            logger.debug(
-                "BLE address changed from %s to %s; closing connection",
-                self.device.address,
-                device.address,
-            )
-            async_create_task(self.close())
+        original_device = self.device
         self.device = device
+        if not original_device or not ble_device_has_changed(original_device, device):
+            return
+        original_path: str | None = None
+        new_path: str | None = None
+        if (
+            isinstance(original_device.details, dict)
+            and isinstance(device.details, dict)
+            and "path" in original_device.details
+            and "path" in device.details
+        ):
+            original_path = original_device.details["path"]
+            new_path = device.details["path"]
+        logger.debug(
+            "%s: BLE device changed from %s (%s) [rssi:%s] to %s (%s) [rssi:%s]; closing connection",
+            self.name,
+            original_device.address,
+            original_path,
+            original_device.rssi,
+            device.address,
+            new_path,
+            device.rssi,
+        )
+        async_create_task(self.close_after_operation(clear_cached_services=True))
 
     def _async_description_update(
         self, description: HomeKitAdvertisement | None
@@ -238,37 +262,7 @@ class BlePairing(AbstractPairing):
                 description,
             )
 
-        repopulate_accessories = False
-        if description and self.description:
-            if description.config_num > self.description.config_num:
-                logger.debug(
-                    "%s: Config number has changed from %s to %s; char cache invalid",
-                    self.name,
-                    self.description.config_num,
-                    description.config_num,
-                )
-                repopulate_accessories = True
-
-            elif description.state_num != self.description.state_num:
-                # Only process disconnected events if the config number has
-                # not also changed since we will do a full repopulation
-                # of the accessories anyway when the config number changes.
-                #
-                # Otherwise, if only the state number we trigger a poll.
-                #
-                # The number will eventually roll over
-                # so we don't want to use a > comparison here. Also, its
-                # safer to poll the device again to get the latest state
-                # as we don't want to miss events.
-                logger.debug(
-                    "%s: Disconnected event notification received; Triggering catch-up poll",
-                    self.name,
-                )
-                async_create_task(self._async_process_disconnected_events())
-
         super()._async_description_update(description)
-        if repopulate_accessories:
-            async_create_task(self._async_process_config_changed())
 
     async def _async_request(
         self, opcode: OpCode, char: Characteristic, data: bytes | None = None
@@ -332,6 +326,7 @@ class BlePairing(AbstractPairing):
                 self.name,
                 self._async_disconnected,
                 cached_services=self._cached_services,
+                ble_device_callback=lambda: self.device,
             )
             self._cached_services = self.client.services
             logger.debug(
@@ -406,18 +401,7 @@ class BlePairing(AbstractPairing):
             self._session_id = session_id
             self._derive = derive
 
-    async def _async_process_config_changed(self) -> None:
-        """Handle config changed seen from the advertisement."""
-        try:
-            await self._populate_accessories_and_characteristics()
-        except (
-            AccessoryDisconnectedError,
-            *BLEAK_EXCEPTIONS,
-            AccessoryNotFoundError,
-        ) as exc:
-            logger.warning("%s: Failed to process config change: %s", self.name, exc)
-
-    async def _async_process_disconnected_events(self) -> None:
+    async def _process_disconnected_events(self) -> None:
         """Handle disconnected events seen from the advertisement."""
         logger.debug(
             "%s: Polling subscriptions for changes during disconnection; rssi=%s",
@@ -502,11 +486,18 @@ class BlePairing(AbstractPairing):
 
         return accessories
 
-    async def close(self) -> None:
-        async with self._connection_lock:
-            await self._close_while_locked()
+    @operation_lock
+    async def close_after_operation(self, clear_cached_services: bool = False) -> None:
+        """Close the client after an operation."""
+        await self.close(clear_cached_services)
 
-    async def _close_while_locked(self):
+    async def close(self, clear_cached_services: bool = False) -> None:
+        async with self._connection_lock:
+            await self._close_while_locked(clear_cached_services)
+
+    async def _close_while_locked(self, clear_cached_services: bool = False):
+        if clear_cached_services:
+            self._cached_services = None
         if not self.client or not self.client.is_connected:
             return
         try:
@@ -541,6 +532,8 @@ class BlePairing(AbstractPairing):
             ):
                 continue
             for char in service.characteristics:
+                if char.type in WRITE_FIRST_REQUIRED_CHARACTERISTICS:
+                    continue
                 if CharacteristicPermissions.paired_read not in char.perms:
                     continue
                 chars.append(char)
@@ -594,9 +587,6 @@ class BlePairing(AbstractPairing):
                 # and we are not forcing an update
                 return
 
-            if not self.accessories:
-                self._load_accessories_from_cache()
-
             update_values = force_update
             config_changed = False
             if self.description:
@@ -744,12 +734,35 @@ class BlePairing(AbstractPairing):
 
         async with self._ble_request_lock:
             for char in characteristics:
-                data = await self._async_request_under_lock(OpCode.CHAR_READ, char)
+                if char.type in WRITE_FIRST_REQUIRED_CHARACTERISTICS:
+                    logger.debug(
+                        "%s: Ignoring write first only characteristic %s",
+                        self.name,
+                        char.iid,
+                    )
+                    continue
+
+                logger.debug("%s: Reading characteristic %s", self.name, char.type)
+
+                try:
+                    data = await self._async_request_under_lock(OpCode.CHAR_READ, char)
+                except PDUStatusError as ex:
+                    # For the apple defines ones we know about we can avoid triggering
+                    # this state, but we do not know all the vendor custom chars so
+                    # we need to skip in this case.
+                    if ex.status == PDUStatus.INVALID_REQUEST:
+                        logger.debug(
+                            "%s: Reading characteristic %s resulted in an invalid request (skipped)",
+                            self.name,
+                            char.type,
+                        )
+                        continue
                 decoded = dict(TLV.decode_bytes(data))[1]
 
                 logger.debug(
-                    "%s: Read characteristic got data, expected format is %s: data=%s decoded=%s",
+                    "%s: Read characteristic %s got data, expected format is %s: data=%s decoded=%s",
                     self.name,
+                    char.type,
                     char.format,
                     data,
                     decoded,
